require 'spec_helper'

describe Environment do
  let(:project) { create(:project, :stubbed_repository) }
  let(:environment) { create(:environment, project: project) }

  describe '#pod_names' do
    context 'when environment does not have a rollout status' do
      it 'returns an empty array' do
        expect(environment.pod_names).to eq([])
      end
    end

    context 'when environment has a rollout status' do
      it 'returns the pod_names' do
        pod_name = "pod_1"
        create(:cluster, :provided_by_gcp, environment_scope: '*', projects: [project])
        create(:deployment, environment: environment)

        allow_any_instance_of(Gitlab::Kubernetes::RolloutStatus).to receive(:instances)
          .and_return([{ pod_name: pod_name }])

        expect(environment.pod_names).to eq([pod_name])
      end
    end
  end

  describe '#protected?' do
    subject { environment.protected? }

    before do
      stub_feature_flags(protected_environments: enabled)
    end

    context 'when Protected Environments feature is not on' do
      let(:enabled) { false }

      it { is_expected.to be_falsy }
    end

    context 'when Protected Environments feature is on' do
      let(:enabled) { true }

      context 'when Protected Environments feature is not available in the project' do
        it { is_expected.to be_falsy }
      end

      context 'when Protected Environments feature is available in the project' do
        before do
          allow(project).to receive(:feature_available?)
            .with(:protected_environments).and_return(true)
        end

        context 'when the environment is protected' do
          before do
            create(:protected_environment,  name: environment.name, project: project)
          end

          it { is_expected.to be_truthy }
        end

        context 'when the environment is not protected' do
          it { is_expected.to be_falsy }
        end
      end
    end
  end

  describe '#protected_deployable_by_user?' do
    let(:user) { create(:user) }
    let(:protected_environment) { create(:protected_environment, :maintainers_can_deploy, name: environment.name, project: project) }

    subject { environment.protected_deployable_by_user?(user) }

    before do
      stub_feature_flags(protected_environments: enabled)
    end

    context 'when Protected Environments feature is not on' do
      let(:enabled) { false }

      it { is_expected.to be_truthy }
    end

    context 'when Protected Environments feature is on' do
      let(:enabled) { true }

      context 'when Protected Environments feature is available in the project' do
        before do
          allow(project).to receive(:feature_available?)
            .with(:protected_environments).and_return(true)
        end

        context 'when the environment is not protected' do
          it { is_expected.to be_truthy }
        end

        context 'when environment is protected and user dont have access to it' do
          before do
            protected_environment
          end

          it { is_expected.to be_falsy }
        end

        context 'when environment is protected and user have access to it' do
          before do
            protected_environment.deploy_access_levels.create(user: user)
          end

          it { is_expected.to be_truthy }
        end
      end
    end
  end
end
